<?php
/** 
 * 
 * @version 1.0.2
 * @author Piotr Kozdęba <piotr.ko@modulesgarden.com>
 */

//SOME DEFINES
defined('DS') ? null : define('DS',DIRECTORY_SEPARATOR);

class OPO_Autoloader
{
    private static $directory = '';
    private static $available_classes = '';
    private static $class_prefix = '';

    public static function register($directory = '', $class_prefix = '')
    {
        
        if(trim($directory) == '')
        {
            self::$directory = dirname(__FILE__).DS;
        }
        else
        {
            self::$directory = $directory;
        }
        
        if($class_prefix != '') { self::$class_prefix = $class_prefix; }
    
        
        self::$available_classes = array();
        
        $di = new RecursiveDirectoryIterator(self::$directory);
        foreach (new RecursiveIteratorIterator($di) as $filename => $file)
        {
            self::$available_classes[$filename] = $filename;
        }
        
        spl_autoload_register(array('OPO_Autoloader', 'autoload'));
        
        return TRUE;
        
    }
    
    public static function unregister()
    {
        
        spl_autoload_unregister(array('OPO_Autoloader', 'autoload'));
        
        return TRUE;
        
    }
    
    public static function autoload($class_name,$debug=false) 
    {
        
        $parts = explode('\\', $class_name);
        $class = end($parts);                    
        
        //creating path to class:
        $parts2 = explode('_', $class);
        
        if (!empty($parts2))
        {
            if ($parts2[0] == self::$class_prefix) { array_shift($parts2); }
            $parts2[count($parts2)-1] = 'class.'.$parts2[count($parts2)-1];
        }
        
        $file_name = self::$directory.  strtolower(implode(DS,$parts2)).'.php';
        
        if ($debug) { return '[['.$file_name.']]<hr>'.print_r(self::$available_classes,true); } 
        
        if(isset(self::$available_classes[$file_name]))
        {
            require_once ($file_name);
        }
        else 
        {
            //searching in basic subfolder:
            $file_name = self::$directory.'basic'.DS.strtolower(implode(DS,$parts2)).'.php';
            if(isset(self::$available_classes[$file_name]))
            {
                require_once ($file_name);
            }
        }
    }
    
}

