<?php
/** 
 * 
 * @version 1.0.0
 * @author Piotr Kozdęba <piotr.ko@modulesgarden.com>
 */
class OPO_WHMCS_Products
{
    
    /**
     * 
     * @param int       $rowid - id of row (could be an array with ids). If null or array it will return array of results ( eg. array(rowid=>value)) )
     * @param string    $fieldtoreturn - null (default) to get full field info despite of only $fieldtoreturn value
     * @return mixed FALSE if there is no such row
     */
    public static function get($rowid, $fieldtoreturn = null)
    {
        $db = new OPO_Database_Database();
        $rowid_query = array();
        if (null!==$rowid && is_array($rowid))
        {
            foreach($rowid as $id)
            {
                $rowid_query []= $db->escape($id);
            }
        }
        
        $valdb = $db->query(
            "SELECT ".((null!==$fieldtoreturn && (null===$rowid || is_array($rowid)))?"`id`":"").((null!==$fieldtoreturn && ($fieldtoreturn!='id' || (null!==$rowid && !is_array($rowid))))?"`".$fieldtoreturn."`":"*")." FROM `tblproducts`".
            ($rowid?" WHERE ".(is_array($rowid)?"`id` IN (".implode(',',$rowid_query).")":"`id`=".$db->escape($rowid)):"")
        );
       
        if ($rowid!==null && !is_array($rowid))
        {
            if (null===$fieldtoreturn)  
            { 
                $val = $valdb->get();
            }
            else                       
            {
                $val = $valdb->get($fieldtoreturn);
            }
        }
        
        if ($rowid===null || is_array($rowid))
        {
            $val_unprep = $valdb->asArray();
            $val = array();
            foreach ($val_unprep as $row)
            {
                $val[$row['id']] = (null===$fieldtoreturn)?$row:$row[$fieldtoreturn];
            
            }
        }
        
        return $val;//($val!==FALSE?$val:FALSE); 
    }
    
    /**
     * 
     * @param int       $rowid - id of row (could be an array with ids). WARNING! If null it will update every row
     * @param array     $settings  - array with fields to set in format array('fieldname'=>value). if null or empty the method will return FALSE and nothing will be updated.
     * @return mixed FALSE if there is no such row
     */
    public static function set($rowid,$settings)
    {
        $db = new OPO_Database_Database();
        
        if (null===$settings || empty($settings) || !is_array($settings))
        {
            return FALSE;
        }
        
        //generating update string:
        $update_string_tab = array();
        foreach ($settings as $key=>$value)
        {
            $update_string_tab []= '`'.$key.'`='.$db->escape($value);
        }
        $update_string = implode(', ',$update_string_tab);
        
        
        $rowid_query = array();
        if (null!==$rowid && is_array($rowid))
        {
            foreach($rowid as $id)
            {
                $rowid_query []= $db->escape($id);
            }
        }
        
        $val = $db->query(
            "UPDATE `tblproducts`".
            "SET ".$update_string.
            ($rowid?" WHERE ".(is_array($rowid)?"`id` IN (".implode(',',$rowid_query).")":"`id`=".$db->escape($rowid)):"")
        )->affectedRows();
        
        
        return ($val>0?TRUE:FALSE);
    }
    
    /**
     * 
     * @param int    $rowid - id of row. WARNING! Cannot be null or multiple ids in array
     * @param string $fieldname - name of field to increase its value
     * @param mixed  $delta - value that will be added
     * @param mixed  $limit - if null then no limit
     * @return mixed FALSE if there is no such row or return added value. Warning! It can return 0 if there is field but its value cannot be changed!
     */
    public static function increase($rowid,$fieldname,$delta,$limit)
    {
        $db = new OPO_Database_Database();
        $valx=0;
        
        $db->query('BEGIN');
        
        $oldval = $db->query(
            "SELECT `".$fieldname."`".
            " FROM `tblproducts`".
            " WHERE `id`=".$db->escape($rowid)
        )->get($fieldname);
        
        if ($limit!==null)
        {
            $valx = $db->query(
                "UPDATE `tblproducts`". 
                " SET `".$fieldname."`= ".$db->escape($limit)." WHERE `id`=".$db->escape($rowid)." AND `".$fieldname."` + ".$db->escape($delta)." > ".$db->escape($limit)
            )->affectedRows();
        }
        
        $val = $db->query(
            "UPDATE `tblproducts`". 
            " SET `".$fieldname."`= `".$fieldname."` + ".$db->escape($delta)." WHERE `id`=".$db->escape($rowid).($limit !== null?" AND `".$fieldname."` + ".$db->escape($delta)." <= ".$db->escape($limit):"")
        )->affectedRows();
        
        $db->query('COMMIT');
        
        $valchange=0;
        if ($valx>0) $valchange = $limit-$oldval;
        elseif ($val>0) $valchange = $delta;
        
        return ($oldval!==FALSE?$valchange:FALSE);
    }    
    
    /**
     * Alias of increase(...) method.
     * 
     * @param int    $rowid - id of row. WARNING! Cannot be null or multiple ids in array
     * @param string $fieldname - name of field to increase its value
     * @param mixed  $delta - value that will be added
     * @param mixed  $limit - if null then no limit
     * @return mixed FALSE if there is no such row or return added value. Warning! It can return 0 if there is field but its value cannot be changed!
     */
    public static function inc($rowid,$fieldname,$delta,$limit)
    {
        self::increase($rowid,$fieldname,$delta,$limit);
    }
        
    /**
     * 
     * @param int    $rowid - id of row. WARNING! Cannot be null or multiple ids in array
     * @param string $fieldname - name of field to decrease its value
     * @param mixed  $delta - value that will be subtracted
     * @param mixed  $limit - if null then no limit
     * @return mixed FALSE if there is no such row or return subtracted value. Warning! It can return 0 if there is field but its value cannot be changed!
     */
    public static function decrease($rowid,$fieldname,$delta,$limit)
    {
        $db = new OPO_Database_Database();
        $valx=0;
        
        $db->query('BEGIN');
        
        $oldval = $db->query(
            "SELECT `".$fieldname."`".
            " FROM `tblproducts`".
            " WHERE `id`=".$db->escape($rowid)
        )->get($fieldname);
        
        if ($limit!==null)
        {
            $valx = $db->query(
                "UPDATE `tblproducts`". 
                " SET `".$fieldname."`= ".$db->escape($limit)." WHERE `id`=".$db->escape($rowid)." AND `".$fieldname."` - ".$db->escape($delta)." < ".$db->escape($limit)
            )->affectedRows();
        }
        
        $val = $db->query(
            "UPDATE `tblproducts`". 
            " SET `".$fieldname."`= `".$fieldname."` - ".$db->escape($delta)." WHERE `id`=".$db->escape($rowid).($limit !== null?" AND `".$fieldname."` - ".$db->escape($delta)." >= ".$db->escape($limit):"")
        )->affectedRows();
        
        $db->query('COMMIT');
        
        $valchange=0;
        if ($valx>0) $valchange = -($limit-$oldval);
        elseif ($val>0) $valchange = $delta;
        
        return ($oldval!==FALSE?$valchange:FALSE);
    } 
    
    /**
     * Alias of decrease(...) method.
     * 
     * @param int    $rowid - id of row. WARNING! Cannot be null or multiple ids in array
     * @param string $fieldname - name of field to decrease its value
     * @param mixed  $delta - value that will be subtracted
     * @param mixed  $limit - if null then no limit
     * @return mixed FALSE if there is no such row or return subtracted value. Warning! It can return 0 if there is field but its value cannot be changed!
     */
    public static function dec($rowid,$fieldname,$delta,$limit)
    {
        self::decrease($rowid,$fieldname,$delta,$limit);
    }
    
    //create is not supported - it is better to use whmcs internal api calls or other whmcs functionality to achieve creation
    
}

